<?php
/**
 * Diafan.CMS
 *
 * @author     diafan.ru
 * @version    4.4
 * @license http://cms.diafan.ru/license.html
 * @copyright  Copyright (c) 2003-2011 OOO "Диафан". (http://diafan.ru)
 */

if(! defined('DIAFAN'))
{
	echo 'diafan.CMS version 4.4.1.10';
	exit;
}

/**
 * Init
 *
 * Основной класс системы
 */
class Init extends Core
{
	/**
	 * @var integer номер текущей страницы
	 */
	public $cid;

	/**
	 * @var string ЧПУ текущей страницы сайта
	 */
	public $rewrite;

	/**
	 * @var string ЧПУ текущей страницы, сгенерированный модулем
	 */
	public $rewrite_module;

	/**
	 * @var string название текущей страницы
	 */
	public $name;

	/**
	 * @var string заголовок текущей страницы для тега title
	 */
	public $title_meta;

	/**
	 * @var string заголовок страницы, сформированный прикрепленным модулем
	 */
	public $titlemodule;

	/**
	 * @var array данные для редактирования заголовка модуля
	 */
	public $edit_titlemodule;

	/**
	 * @var string заголовок текущей страницы для тега title, сформированный прикрепленным модулем
	 */
	public $titlemodule_meta;

	/**
	 * @var string ключевые слова текущей страницы
	 */
	public $keywords;

	/**
	 * @var string спецтекст текущей страницы
	 */
	public $special;

	/**
	 * @var string описание текущей страницы
	 */
	public $descr;

	/**
	 * @var string файл шаблона текущей страницы
	 */
	public $theme;

	/**
	 * @var string контент текущей страницы
	 */
	public $text;

	/**
	 * @var integer номер страницы-родителя
	 */
	public $parent_id;

	/**
	 * @var boolean спрятать ссылки на предыдущую, последующую страницы
	 */
	public $hide_previous_next;

	/**
	 * @var boolean не показывать заголовок h1 текущей страницы
	 */
	public $title_no_show;

	/**
	 * @var integer время редактирования текущей страницы
	 */
	public $timeedit;

	/**
	 * @var string модуль, прикрепленный к текущей странице
	 */
	public $module;

	/**
	 * @var array часть навигации "Хлебные крошки", сформированная прикрепленным модулем
	 */
	public $path;

	/**
	 * @var array переменные, передаваемые в URL
	 */
	public $rewrite_variable_names_all = array('cat', 'param', 'show', 'page', 'comments', 'year', 'month', 'step', 'sort', 'add', 'edit');

	/**
	 * @var array переменные, прикрепленные к псевдоссылке текущей страницы
	 */
	public $rewrite_variable_in_page = array();

	/**
	 * @var array переменные, передаваемые в URL для текущего модуля
	 */
	public $rewrite_variable_names = array();

	/**
	 * @var integer номер страницы
	 */
	public $page;

	/**
	 * @var integer переменная, передаваемая в URL: номер элемента
	 */
	public $show;

	/**
	 * @var integer переменная, передаваемая в URL: номер параметра
	 */
	public $param;

	/**
	 * @var integer переменная, передаваемая в URL: номер страницы с комментариями
	 */
	public $comments;

	/**
	 * @var integer переменная, передаваемая в URL: номер категории
	 */
	public $cat;

	/**
	 * @var integer переменная, передаваемая в URL: год
	 */
	public $year;

	/**
	 * @var integer переменная, передаваемая в URL: месяц
	 */
	public $month;

	/**
	 * @var integer переменная, передаваемая в URL: номер шага
	 */
	public $step;

	/**
	 * @var integer переменная, передаваемая в URL: направление сортировки
	 */
	public $sort;

	/**
	 * @var integer переменная, передаваемая в URL: добавить элемент (1)
	 */
	public $add;

	/**
	 * @var integer переменная, передаваемая в URL: номер редактируемого элемента
	 */
	public $edit;

	/**
	 * @var object пользователи
	 */
	public $_user;

	/**
	 * @var object кэширование
	 */
	public $_cache;

	/**
	 * @var object ЧПУ
	 */
	public $_route;

	/**
	 * @var object шаблоны
	 */
	public $_tpl;

	/**
	 * @var array объекты модулей
	 */
	private $_objects;

	/**
	 * Конструктор класса. Определяет свойства класса
	 *
	 * @return void
	 */
	public function __construct()
	{
		require_once(ABSOLUTE_PATH.'includes/user.php');
		$this->_user = new User();

		require_once(ABSOLUTE_PATH.'includes/cache.php');
		$this->_cache = new Cache;

		require_once(ABSOLUTE_PATH.'includes/route.php');
		$this->_route = new Route($this);

		require_once(ABSOLUTE_PATH.'includes/template.php');
		$this->_tpl = new Template($this);
	}

	/**
	 * Подключает вспомогательные модули, если они не подключены
	 *
	 * @return boolean true
	 */
	public function __get($name)
	{
		if(! isset($this->_objects[$name]))
		{
			// подключаем обработчик запросов модуля
			if(substr($name, 0, 1) == '_')
			{
				$module = substr($name, 1);
				if(file_exists(ABSOLUTE_PATH.'modules/'.$module.'/'.$module.'.inc.php'))
				{
					include_once(ABSOLUTE_PATH.'modules/'.$module.'/'.$module.'.inc.php');
					$class = ucfirst($module).'_inc';
					$this->_objects[$name] = new $class($this);
				}
			}
			else
			{
				return '';
			}
		}
		return $this->_objects[$name];
	}

	/**
	 * Инициализирует генерирование страницы
	 *
	 * @return boolean true
	 */
	public function start()
	{
		Dev::set_profiling();

		require_once(ABSOLUTE_PATH.'includes/session.php');
		Session::init();

		$this->language_all();
		$this->magic_quote_off();
		$this->get_user();

		$this->set_timezone();
		$this->get_language_site();

		$this->include_other_file();

		$this->prepare_rewrite();
		$this->get_page_in_site();

		require_once(ABSOLUTE_PATH.'includes/gzip.php');
		Gzip::init_gzip();

		$module = $this->get_module();

		include_once(ABSOLUTE_PATH.'includes/parser_theme.php');
		$parser = new Parser_theme($this, $module);

		$this->get_headers();
		$parser->show_theme();

		Dev::get_profiling();

		Gzip::do_gzip();
		return TRUE;
	}

	/**
	 * Подключает файлы для решения нестандартых задач (вывод каптчи, вывод прикрепленных файлов...)
	 *
	 * @return boolean true
	 */
	private function include_other_file()
	{
		if(! empty($_GET["rewrite"]))
		{
			$rewrite_array = explode("/", $_GET["rewrite"]);
			if(count($rewrite_array) > 1 && ! in_array($rewrite_array[1], array('model', 'view', 'admin', 'inc', 'ajax', 'sitemap'))
			   && file_exists(ABSOLUTE_PATH.'modules/'.$rewrite_array[0].'/'.$rewrite_array[0].'.'.$rewrite_array[1].'.php'))
			{
				$path = ABSOLUTE_PATH.'modules/'.$rewrite_array[0].'/'.$rewrite_array[0].'.'.$rewrite_array[1].'.php';
				unset($rewrite_array[0]);
				unset($rewrite_array[1]);
				$_GET["rewrite"] = implode('/', $rewrite_array);
				include_once($path);
			}
		}

		if(strstr($_GET["rewrite"], 'sitemap.xml') !== FALSE)
		{
		    include_once(ABSOLUTE_PATH.'includes/sitemap.xml.php');
		}
		return TRUE;
	}

	/**
	 * Подготавливает запрос для идентифицикации страницы в таблице {site} по rewrite или по id,
	 * удаляет из строки запроса $_GET[rewrite] переданные переменные
	 *
	 * @return boolean true
	 */
	private function prepare_rewrite()
	{
		$arguments_in_url = FALSE;
		if($this->get_rewrite($_GET["rewrite"], $arguments_in_url))
		{
			return TRUE;
		}
		if($_GET["rewrite"])
		{
			$rewrite_array = explode("/",$_GET["rewrite"]);
			foreach($this->rewrite_variable_names_all as $name)
			{
				$this->$name = 0;
			}

			foreach($rewrite_array as $key => $ra)
			{
				foreach($this->rewrite_variable_names_all as $name)
				{
					if(preg_match('/'.$name.'([0-9]+)/', $ra, $result))
					{
						$this->$name = $result[1];
						unset($rewrite_array[$key]);
						$arguments_in_url = TRUE;
					}
				}
			}

			$_GET["rewrite"] = implode("/", $rewrite_array);
			if(! $this->get_rewrite($_GET["rewrite"], $arguments_in_url))
			{
				include(ABSOLUTE_PATH.'includes/404.php');
			}
		}

		if(! $this->rewrite)
		{
			$this->cid = empty($_GET["url"]) ? 1 : intval($_GET["url"]);
		}
		return TRUE;
	}

	/**
	 * Получаем страницу по псевдоссылке
	 *
	 * @param string $rewrite текущая псевдоссылка
	 * @param boolean $arguments_in_url в URL переданы аргументы
	 * @return boolean
	 */
	public function get_rewrite($rewrite, $arguments_in_url)
	{
		if($row = $this->_route->search($rewrite, $arguments_in_url))
		{
			if(! $this->show && ! $this->cat)
			{
				$this->rewrite_module = $row["rewrite"];
			}
			if($row["element_id"])
			{
				if($this->show)
				{
					include(ABSOLUTE_PATH.'includes/404.php');
				}
				$this->show =  $row["element_id"];
				$this->rewrite_variable_in_page[] = 'show';
			}
			if($row["cat_id"])
			{
				if($this->cat)
				{
					include(ABSOLUTE_PATH.'includes/404.php');
				}
				$this->cat =  $row["cat_id"];
				$this->rewrite_variable_in_page[] = 'cat';
			}
			if($row["param_id"])
			{
				if($this->param)
				{
					include(ABSOLUTE_PATH.'includes/404.php');
				}
				$this->param = $row["param_id"];
				$this->rewrite_variable_in_page[] = 'param';
			}
			$this->cid     = $row["site_id"];
			$this->module  = $row["module_name"] ? $row["module_name"] : $this->module;
			$this->rewrite = $row["rewrite"];
			return TRUE;
		}
		return FALSE;
	}

	/**
	 * Идентифицирует страницу сайта, задает параметры страницы
	 *
	 * @return boolean true
	 */
	private function get_page_in_site()
	{
		if($this->cid)
		{
			$current_page = DB::fetch_array(DB::query(
					"SELECT id, parent_id, [name], [title_meta], [name], [keywords], [special], [descr], title_no_show, [text], "
					." timeedit, theme, addmodule, sort FROM {site}"
					." WHERE [act]='1' AND trash='0' AND block='0'"
					.(! $this->_user->id ? " AND access='0'" : '')
					." AND id=%d ORDER BY sort ASC, id ASC", $this->cid
				));
			if(empty($current_page))
			{
				include(ABSOLUTE_PATH.'includes/404.php');
			}
			if(! $current_page["theme"])
			{
				$current_page["theme"] = $this->search_theme($current_page["id"]);
			}
		}
		else
		{
			include(ABSOLUTE_PATH.'includes/404.php');
		}

		$this->cid           = $current_page["id"];
		$this->name          = $current_page['name'];
		$this->keywords      = $current_page['keywords'];
		$this->special       = $current_page['special'];
		$this->descr         = $current_page['descr'];
		$this->text          = $current_page['text'];
		$this->parent_id     = $current_page['parent_id'];
		$this->title_no_show = $current_page['title_no_show'];
		$this->title_meta    = $current_page['title_meta'];
		$this->timeedit      = $current_page['timeedit'];
		$this->theme         = $current_page['theme'];
		$this->module        = $current_page["addmodule"];

		return TRUE;
	}

	/**
	 * Отдает браузеру заголовки страницы
	 *
	 * @return boolean true
	 */
	private function get_headers()
	{
		header("Expires: ".date("r"));
		header("Last-Modified: ".(! $this->timeedit ? gmdate("D, d M Y H:i:s") : gmdate("D, d M Y H:i:s", $this->timeedit)) . " GMT");
		header("Cache-Control: no-store, no-cache, must-revalidate");
		header("Pragma: no-cache");
		header("Cache-Control: post-check=0, pre-check=0", FALSE);
		header('Content-Type: text/html; charset=utf-8');
		return TRUE;
	}

	/**
	 * Подключает модуль, прикрепленный к странице
	 *
	 * @return object
	 */
	private function get_module()
	{
		// подключаем обработчик запросов модуля
		if(! empty($_POST["module"]))
		{
			$module = preg_replace('/[^a-z0-9_]+/', '', $_POST["module"]);
			if(file_exists(ABSOLUTE_PATH.'modules/'.$module.'/'.$module.'.ajax.php'))
			{
				include_once(ABSOLUTE_PATH.'modules/'.$module.'/'.$module.'.ajax.php');
				$class = ucfirst($module).'_ajax';
				$ajax = new $class($this);
				if($ajax->ajax_request())
				{
					exit;
				}
			}
		}

		if($this->module && file_exists(ABSOLUTE_PATH.'modules/'.$this->module.'/'.$this->module.'.php'))
		{
			include_once(ABSOLUTE_PATH.'modules/'.$this->module.'/'.$this->module.'.php');
			if(file_exists(ABSOLUTE_PATH.'modules/'.$this->module.'/'.$this->module.'.model.php'))
			{
				include_once(ABSOLUTE_PATH.'modules/'.$this->module.'/'.$this->module.'.model.php');
			}
			$name_class_module = ucfirst($this->module);
			$mod = new $name_class_module($this);
			$mod->init();

			foreach($this->rewrite_variable_names_all as $name)
			{
				if($this->$name && ! in_array($name, $mod->rewrite_variable_names))
				{
					include(ABSOLUTE_PATH.'includes/404.php');
				}
			}
			return $mod;
		}
		else
		{
			foreach($this->rewrite_variable_names_all as $name)
			{
				if($this->$name && ($name != "comments" || ! $this->configmodules("comments", "site")))
				{
					include(ABSOLUTE_PATH.'includes/404.php');
				}
			}
			$mod = new Controller($this);
			return $mod;
		}
	}

	/**
	 * Записывает данные о языках сайта
	 *
	 * @return boolean true
	 */
	private function get_language_site()
	{
		//при авторизации язык сайта изменяем в соответствии с настройками пользователя
		if(! empty($_GET["rewrite"]) && preg_match("/^lang/", $_GET["rewrite"]))
		{
			$_GET["rewrite"] = preg_replace("/^lang\//", '', $_GET["rewrite"]);
			$_GET["rewrite"] = preg_replace("/^lang/", '', $_GET["rewrite"]);
			define('_LANG', $this->_user->language);
		}

		if($_GET["rewrite"] || defined('_LANG') && _LANG)
		{
			$rew = explode('/', $_GET["rewrite"], 2);
			foreach($this->language_all as $l => $langarr)
			{
				if(! defined('_LANG') && in_array($rew[0], $langarr) || defined('_LANG') && $langarr[0] == _LANG)
				{
					$_GET["rewrite"] = preg_replace('/^'.$rew[0].'(\/)*/', '', $_GET["rewrite"]);
					if(! defined('_LANG'))
					{
						define('_LANG', $langarr[0]);
					}
					define('BASE_PATH_HREF', BASE_PATH.$langarr[1].'/');
					define('BASE_URL', $_SERVER["HTTP_HOST"].(REVATIVE_PATH ? '/'.REVATIVE_PATH : '').'/'.$langarr[1]);
					if(defined('TIT'.$langarr[0]))
					{
						define('TITLE', constant('TIT'.$langarr[0]));
					}
					else
					{
						define('TITLE', TIT);
					}
					include_once(ABSOLUTE_PATH.'language/'.$langarr[1].'.php');
					break;
				}
			}
		}

		if(! defined('_LANG'))
		{
			define('_LANG', '');
			define('TITLE', TIT);
			define('BASE_PATH_HREF', BASE_PATH);
			define('BASE_URL', $_SERVER["HTTP_HOST"].(REVATIVE_PATH ? '/'.REVATIVE_PATH : ''));
			include_once(ABSOLUTE_PATH.'language/'.LANGUAGE_BASE.'.php');
		}
		return TRUE;
	}

	/**
	 * Борьба с магическими кавычками
	 *
	 * @return boolean true
	 */
	private function magic_quote_off()
	{
		if(get_magic_quotes_gpc())
		{
			$_GET = $this->stripslashes_array($_GET);
			$_POST = $this->stripslashes_array($_POST);
		}
		return TRUE;
	}

	/**
	 * Вырезает магические кавычки из массива
	 *
	 * @return array
	 */
	private function stripslashes_array($array)
	{
		if(is_array($array))
		{
			foreach($array as $key => $value)
			{
				$array[$key] = $this->stripslashes_array($value);
			}
			return $array;
		}
		else
		{
			return stripslashes($array);
		}
	}

	/**
	 * Инициирует авторизацию или выход пользователя из системы
	 *
	 * @return boolean true
	 */
	private function get_user()
	{
		if(! empty($_POST['action']) && $_POST['action'] == 'auth')
		{
			$this->_user->auth($_POST);
		}
		elseif(strstr($_GET["rewrite"], "logout"))
		{
			$this->_user->logout();
		}
		return TRUE;
	}

	/**
	 * Устанавливает часовой пояс
	 *
	 * @return boolean true
	 */
	private function set_timezone()
	{
		if(! TIMEZONE || @ ! date_default_timezone_set(TIMEZONE))
		{
			date_default_timezone_set('Europe/Moscow');
		}
		return TRUE;
	}
}