<?php
/**
 * Diafan.CMS
 *
 * @author     diafan.ru
 * @version    4.4
 * @license http://cms.diafan.ru/license.html
 * @copyright  Copyright (c) 2003-2011 OOO "Диафан". (http://diafan.ru)
 */

if(! defined('DIAFAN'))
{
	include(dirname(dirname(dirname(__FILE__))).'/includes/404.php');
}

/**
 * Shop_admin
 *
 * Редактирование администраторов сайта
 */
class Shop_admin extends Frame_admin
{
	/**
	 * @var string название файла. используется в имени констант, определяющих названия полей
	 */
	public $file = 'shopelement';

	/**
	 * @var string таблица в базе данных
	 */
	public $table = 'shop';

	/**
	 * Конструктор класса
	 *
	 * @return void
	 */
	public function __construct($diafan)
	{
		$this->diafan = $diafan;

		//поля в базе данных для редактирования
		$this->variable_table = array(
			'name'        => 'text',
			'article'     => 'text',
			'act'         => 'checkbox',
			'price'       => 'floattext',
			'no_buy'      => 'checkbox',
			'images'      => 'module',
			'hr1'         => 'hr',
			'param'       => 'function',
			'hr2'         => 'hr',
			'rel_goods'   => 'function',
			'hr3'         => 'hr',
			'tags'        => 'module',
			'comments'    => 'module',
			'rating'      => 'module',
			'cat_id'      => 'function',
			'hr4'         => 'hr',
			'menu'        => 'module',
			'number'      => 'function',
			'timeedit'    => 'text',
			'title_meta'  => 'text',
			'keywords'    => 'text',
			'descr'       => 'textarea',
			'rewrite'     => 'function',
			'hr5'         => 'hr',
			'site_id'     => 'function',
			'hr6'         => 'hr',
			'sort'        => 'function',
			'hr7'         => 'hr',
			'show_yandex' => 'checkbox',
			'map_no_show' => 'checkbox',
			'yandex'      => 'function',
			'hr8'         => 'hr',
			'anons'       => 'editor',
			'text'        => 'editor',
			'special'     => 'editor'
		);

		//поля, которые прячутся в Дополнительно
		$this->variable_other = array(
			'hr4'         => 1,
			'hr5'         => 1,
			'hr6'         => 1,
			'hr7'         => 1,
			'number'      => 1,
			'title_meta'  => 1,
			'timeedit'    => 1,
			'keywords'    => 1,
			'descr'       => 1,
			'rewrite'     => 1,
			'site_id'     => 1,
			'sort'        => 1,
			'yandex'      => 1,
			'show_yandex' => 1,
			'map_no_show' => 1,
			'menu'        => 1
		);

		//переводимые поля
		$this->language_names = array(
			'name',
			'title_meta',
			'act',
			'keywords',
			'special',
			'descr',
			'text',
			'anons'
		);

		$this->config_module = array(
			'act'              => 1, //показать/скрыть
			'menu'             => 1, //используется в меню
			'del'              => 1, //удалить
			'order'            => 1, //сортируется
			'element_site'     => 1, //модуль может быть прикреплен к нескольким страницам
			'element'          => $this->diafan->configmodules("cat", "shop", $this->diafan->site) ? 1 : 0, //используются группы
			'element_multiple' => $this->diafan->configmodules("cat", "shop", $this->diafan->site) ? 1 : 0, //модуль может быть прикреплен к нескольким группам
			'search_name'      => 1, //искать по названию
			'trash'            => 1, //использовать корзину
			'image'            => 1  //показать фотографию в списке
		);

		//выводить в списке содержание полей:
		$this->config_other_row = array(
			'id',
			'text'
		);

		if(! $this->diafan->configmodules('show_yandex_element', 'shop'))
		{
			unset($this->variable_table["show_yandex"]);
		}
		else
		{
			$this->show_tr_click_checkbox = array('show_yandex' => array('yandex'));
		}

		if(! extension_loaded('gd') && ! extension_loaded('gd2'))
		{
			$this->diafan->error = 7;
		}

		//высота поля для Визуального редактора
		$this->editor_height = array(
			'anons' => 200,
			'text'  => 400,
			'special' => 200
		);
	}

	/**
	 * Выводит список товаров
	 * @return boolean true
	 */
	public function show_module()
	{
		if($this->config_module['element'] && ! $this->not_empty_categories)
		{
			echo '<p class="red">'.sprintf(_ADMIN_SHOP_EMPTY_CATEGORIES, '<a href="'.BASE_PATH_HREF.'shop/category/'.($this->diafan->site ? 'site'.$this->diafan->site.'/' : '').'">', '</a>').'</p>';
		}
		else
		{
			$this->addnew_init(_ADMIN_SHOP_ADD_GOODS);
		}
		echo '<table class="list">';
		$this->list_row();
		echo '</table>';

		return TRUE;
	}

	/**
	 * Выводит цену в списке товаров
	 * @return string
	 */
	protected function other_row_id($row)
	{
		$min_price = 0;
		$max_price = 0;
		$result = DB::query("SELECT price FROM {shop_price} WHERE good_id=%d AND trash='0' ORDER BY price ASC", $row["id"]);
		while ($row = DB::fetch_array($result))
		{
			if(! $min_price)
			{
				$min_price = $row["price"];
			}
			$max_price = $row["price"];
		}
		return '</td><td>'.($min_price
				    ? ($min_price * 1).($max_price != $min_price ? ' - '.($max_price * 1) : '')
				      .' '.$this->diafan->configmodules("currency", "shop")
				    : '');
	}

	/**
	 * Редактирование поля "Цена"
	 * @return boolean true
	 */
	protected function edit_price_module()
	{
		if(! $this->diafan->addnew)
		{
			$result = DB::query("SELECT price, param_id, param_value, count_goods FROM {shop_price} WHERE good_id=%d AND trash='0' ORDER BY param_value ASC", $this->diafan->edit);
			while ($row = DB::fetch_array($result))
			{
				if(! $row["param_id"])
				{
					$this->values["price"] = $row["price"];
					$this->values["count"] = $row["count_goods"];
				}
				else
				{
					$this->values["depend"][$row["param_id"]][$row["param_value"]] = array("price" => $row["price"], "count" => $row["count_goods"]);
				}
			}
		}

		echo '
		<tr'.$this->get_other().(! empty($this->values["depend"]) ? ' class="hide"' : '').'>
			<td class="td_first">
				'.$this->variable_names().'
			</td>
			<td>
				<input type="text" name="'.$this->key.'" size="20" value="'.(! empty($this->values["price"]) ? $this->values["price"] * 1 : '')
				.'" class="inpnum"> ';
			if($this->diafan->configmodules("use_count_goods"))
			{
				echo _LANG_COUNT.'
				<input type="text" name="count" size="20" value="'.(! empty($this->values["count"]) ? $this->values["count"] : 0)
				.'" class="inpnum">';
			}
			echo $this->show_help_row().'
			</td>
		</tr>';

		return TRUE;
	}

	/**
	 * Редактирование поля "Яндекс.Маркет"
	 * @return boolean true
	 */
	protected function edit_yandex_module()
	{
		if(! $this->diafan->configmodules("yandex"))
		{
			return;
		}

		$u = array();

		if(! $this->diafan->addnew)
		{
			if($this->value)
			{
				$conf_arr = explode("\n", $this->value);
				foreach($conf_arr as $c_a)
				{
					list($k, $v) = explode("=", $c_a, 2);
					$u[$k] = $v;
				}
			}
		}

		echo '
		<tr'.$this->get_other().' valign="top">
			<td align="right">'
				.$this->variable_names().$this->show_help_row().'
			</td>
			<td>'
		._ADMIN_SHOPELEMENT_YANDEX_CAT.':<br>
		<input type="text" maxLength="70" name="typePrefix" size="40" class="inptext" value="'.(! empty($u["typePrefix"]) ? $u["typePrefix"] : '').'"><br>'

		.'<br>'._ADMIN_SHOPELEMENT_YANDEX_VENDOR.':<br>
		<input type = "text" maxLength="70" name="vendor" size="40" class="inptext" value="'.(! empty($u["vendor"]) ? $u["vendor"] : '').'"><br>'

		._ADMIN_SHOPELEMENT_YANDEX_MODEL.':<br>
		<input type = "text" maxLength="70" name="model" size="40" class="inptext" value="'.(! empty($u["model"]) ? $u["model"] : '').'"><br>'

		._ADMIN_SHOPELEMENT_YANDEX_VENDORCODE.':<br>
		<input type = "text" maxLength="70" name="vendorCode" size="40" class="inptext" value="'.(! empty($u["vendorCode"]) ? $u["vendorCode"] : '').'"><br>'

		._ADMIN_SHOPELEMENT_YANDEX_SALES_NOTES.':<br>
		<input type = "text" maxLength="50" name="sales_notes" size="40" class="inptext" value="'.(! empty($u["sales_notes"]) ? $u["sales_notes"] : '').'"><br>'

		._ADMIN_SHOPELEMENT_YANDEX_MANUFACTURER_WARRANTY.':
		<input type = "checkbox" name="manufacturer_warranty" value="1"'.(! empty($u["manufacturer_warranty"]) ? ' checked' : '').'><br><br>'

		._ADMIN_SHOPELEMENT_YANDEX_COUNTRY_OF_ORIGINAL.':<br>
		<input type = "text" maxLength="70" name="country_of_origin" size="40" class="inptext" value="'.(! empty($u["country_of_origin"]) ? $u["country_of_origin"] : '').'">
			</td>
		</tr>';

		return TRUE;
	}

	/**
	 * Редактирование поля "Дополнительные параметры"
	 *
	 * @return boolean true
	 */
	protected function edit_param_module()
	{
		echo '
		<tr>
			<td></td>
			<td>
				<a href="'.BASE_PATH_HREF.'shop/param/addnew1/" target="_blank">'._ADMIN_SHOP_ADD_PARAM.'</a>
				'.$this->show_help_row().'
			</td>
		</tr>';
		$values = array();
		$rvalues = array();

		if($this->diafan->addnew)
		{
			$result = DB::query("SELECT p.id, p.name, p.type FROM {shop_param} as p "
			                    .($this->diafan->configmodules("cat", "shop", $this->diafan->site)
			                      ? " INNER JOIN {shop_param_category_rel} as cp ON cp.cat_id IN (%d, 0) AND cp.element_id=p.id" : "")
			                    ." WHERE p.trash='0'".($this->diafan->site ? " AND site_id=".$this->diafan->site : '')
			                    ." GROUP BY p.id ORDER BY p.sort ASC", $this->diafan->cat
			                   );
		}
		else
		{
			$result = DB::query("SELECT p.id, p.name, p.type FROM {shop_param} as p "
			                    .($this->diafan->configmodules("cat", "shop", $this->values["site_id"])
			                      ? " LEFT JOIN {shop_category_rel} as c ON c.element_id=".$this->diafan->edit
			                        ." INNER JOIN {shop_param_category_rel} as cp ON (cp.cat_id=c.cat_id OR cp.cat_id=0) AND cp.element_id=p.id"
			                    : "")
			                    ." WHERE p.trash='0' AND p.site_id=%d GROUP BY p.id"
			                    ." ORDER BY p.sort ASC", $this->values["site_id"]
			                   );
			$result_el = DB::query("SELECT value as rv, [value], param_id FROM {shop_param_element} WHERE element_id=%d", $this->diafan->edit);
			while ($row_el = DB::fetch_array($result_el))
			{
				$values[$row_el["param_id"]][] = $row_el["value"];
				$rvalues[$row_el["param_id"]][] = $row_el["rv"];
			}
		}

		while($row = DB::fetch_array($result))
		{
			$option = array();
			if(in_array($row["type"], array('select', 'multiple')))
			{
				$result_select = DB::query("SELECT [name], id FROM {shop_param_select} WHERE param_id=%d ORDER BY sort ASC", $row["id"]);
				while ($row_select = DB::fetch_array($result_select))
				{
					$option[] = $row_select;
				}
			}
			if($row["type"] == 'text')
			{
				echo '
				<tr>
					<td align="right">'.$row["name"].'</td>
					<td>
						<input type="text" name="param'.$row["id"].'" size="40" value="'.(! empty($values[$row["id"]]) ? str_replace('"', '&quot;', $values[$row["id"]][0]) : '').'">
					</td>
				</tr>';
			}
			elseif($row["type"] == 'title')
			{
				echo '
				<tr>
					<td align="right"><b>'.$row["name"].'</b></td>
					<td></td>
				</tr>';
			}
			elseif($row["type"] == 'date')
			{
				if(! $this->show_datepicker)
				{
					echo '
					<link rel="stylesheet" href="'.BASE_PATH.'css/timecalendar.css" />
					<script type="text/javascript" src="'.BASE_PATH.'js/timecalendar.js"></script>';
					$this->show_datepicker = TRUE;
				}
				echo '
				<tr>
					<td align="right">'.$row["name"].'</td>
					<td>
						<input type="text" id="param'.$row["id"].'" name="param'.$row["id"].'" size="20" value="'.(! empty($rvalues[$row["id"]]) ? str_replace('"', '&quot;', date("d.m.Y", $rvalues[$row["id"]][0])) : '').'" class="timecalendar" showTime="false">
					</td>
				</tr>';
			}
			elseif($row["type"] == 'numtext')
			{
				echo '
				<tr>
					<td align="right">'.$row["name"].'</td>
					<td>
						<input type="text" name="param'.$row["id"].'" size="20" value="'.(! empty($rvalues[$row["id"]]) ? str_replace('"', '&quot;', $rvalues[$row["id"]][0]) : '').'" class="inpnum">
					</td>
				</tr>';
			}
			elseif($row["type"] == 'textarea')
			{
				echo '
				<tr>
					<td align="right">'.$row["name"].'</td>
					<td>
						<textarea name="param'.$row["id"].'" cols="60" rows="5">'.(! empty($values[$row["id"]]) ? str_replace('"', '&quot;', $values[$row["id"]][0]) : '').'</textarea>
					</td>
				</tr>';
			}
			elseif($row["type"] == 'checkbox')
			{
				echo '
				<tr>
					<td align="right">'.$row["name"].'</td>
					<td>
						<input type="checkbox" name="param'.$row["id"].'" value="1"'.(! $this->diafan->addnew && ! empty($rvalues[$row["id"]][0]) ? " checked":'').'>
					</td>
				</tr>';
			}
			elseif($row["type"] == 'select')
			{
				$value = (! empty($rvalues[$row["id"]][0]) ? $rvalues[$row["id"]][0] : '');
				echo '
				<tr>
					<td align="right">'.$row["name"].'</td>
					<td>
						<select name="param'.$row["id"].'">
						<option value="">-</option>';
						foreach($option as $k => $v)
						{
							echo '<option value="'.$v["id"].'"'.($value == $v["id"] ? ' selected' : '').'>'.$v["name"].'</option>';
						}
						echo '</select>
					</td>
				</tr>';
			}
			elseif($row["type"] == 'multiple')
			{
				$value = (! empty($rvalues[$row["id"]]) ? $rvalues[$row["id"]] : array());
				echo '
				<tr>
					<td align="right">
						'.$row["name"].'
						<a href="javascript:void(0)" onClick="show_div(\'help_depend_price'.$row["id"].'\');"><img src="'.BASE_PATH.'adm/img/quest.gif"></a>
						<br>
						<div id="help_depend_price'.$row["id"].'" class="help_row">'._ADMIN_SHOPELEMENT_DEPEND_PRICE_HELP.'</div>
						<div class="check_price'.(! empty($this->values["depend"]) && empty($this->values["depend"][$row["id"]]) ? ' hide' : '').'">
							<input type="checkbox" name="check_price_'.$row["id"].'" value="1"
							'.(! empty($this->values["depend"][$row["id"]]) ? ' checked' : '').' rel="'.$row["id"].'">
							'._ADMIN_SHOP_DEPEND_PRICE.'
						</div>
					</td>
					<td>
						<div class="select_param'.(! empty($this->values["depend"][$row["id"]]) ? ' hide' : '').'" id="select_param_'.$row["id"].'">
						<select name="param'.$row["id"].'[]" multiple="multiple">';
						foreach($option as $k => $v)
						{
							echo '<option value="'.$v["id"].'"'.(in_array($v["id"], $value) ? ' selected' : '').'>'.$v["name"].'</option>';
						}
						echo '</select>
					</div>
					<div class="price_param'.(empty($this->values["depend"][$row["id"]]) ? ' hide' : '').'" id="price_param_'.$row["id"].'">
					<table class="table_in">
					<tr>
						<th>'._ADMIN_VALUE.'</th>
						<th>'._LANG_PRICE.'</th>';
					if($this->diafan->configmodules("use_count_goods"))
					{
						echo '<th>'._LANG_COUNT.'</th>';
					}
					echo '
					</tr>';
					foreach($option as $k => $v)
					{
						echo '<tr>
						<td>
							'.$v["name"].'
						</td>
						<td>
							<input name="param_price_'.$row["id"].'_'.$v["id"].'" value="'
							.(! empty($this->values["depend"][$row["id"]][$v["id"]]) ? $this->values["depend"][$row["id"]][$v["id"]]["price"] * 1 : '').'">';
						if($this->diafan->configmodules("use_count_goods"))
						{
							echo '</td><td>
							<input type="text" name="param_count_'.$row["id"].'_'.$v["id"].'" size="20" value="'.
							(! empty($this->values["depend"][$row["id"]][$v["id"]]) ? $this->values["depend"][$row["id"]][$v["id"]]["count"] : '')
							.'" class="inpnum">';
						}
						echo '
						</td>
						</tr>';
					}
					echo '</table>
					</div>
					</td>
				</tr>';
			}
			elseif($row["type"] == 'editor')
			{
				echo '
				<tr'.$this->get_other().'>
					<td colspan="2">'.$row["name"]
						.'<textarea name="param'.$row["id"].'" style="width:100%;height:200px"'.(HTMLEDITOR ? ' class="htmleditor"' : '').'>'
						.(! $this->diafan->addnew && ! empty($values[$row["id"]]) ? $values[$row["id"]][0] : '')
						.'</textarea>
					</td>
				</tr>';
			}
		}
		return TRUE;
	}

	/**
	 * Редактирование поля "Похожие товары"
	 *
	 * @return boolean true
	 */
	protected function edit_rel_goods_module()
	{
		include_once(ABSOLUTE_PATH.'modules/shop/admin/shop.admin.view.php');
		echo '
		<script type="text/javascript" src="'.BASE_PATH.'modules/shop/admin/shop.admin.js"></script>
		<tr'.$this->get_other().' valign="top">
			<td class="td_first">'.$this->variable_names().'</td>
			<td>
				<div class="rel_goods">';
				if(! $this->diafan->addnew)
				{
					echo Shop_admin_view::rel_goods($this, $this->diafan->edit);
				}
				echo '</div>
				<a href="javascript:void(0)" class="rel_shop_plus" title="'._LANG_ADD.'"><img src="'.BASE_PATH.'adm/img/add.gif" width="16" height="16" alt="'._LANG_ADD.'"></a>
				<div class="hide" id="rel_shop_container"></div>
				'.$this->show_help_row().'
			</td>
		</tr>';
		return TRUE;
	}

	/**
	 * Сохранение поля "Цена"
	 * @return boolean true
	 */
	protected function save_price_module()
	{
		return TRUE;
	}

	/**
	 * Сохранение поля "Похожие товары"
	 * @return boolean true
	 */
	protected function save_rel_goods_module()
	{
		return TRUE;
	}

	/**
	 * Сохранение поля "Яндекс.Маркет"
	 * @return boolean true
	 */
	protected function save_yandex_module()
	{
		if(! $this->diafan->configmodules("yandex"))
		{
			return FALSE;
		}

		$this->query[] = "yandex='%s'";
		$this->value[] =
			'typePrefix='.str_replace("\n", '', $this->diafan->get_param($_POST, "typePrefix", '', 1))."\n"
			.'vendor='.str_replace("\n", '', $this->diafan->get_param($_POST, "vendor", '', 1))."\n"
			.'model='.str_replace("\n", '', $this->diafan->get_param($_POST, "model", '', 1))."\n"
			.'vendorCode='.str_replace("\n", '', $this->diafan->get_param($_POST, "vendorCode", '', 1))."\n"
			.'sales_notes='.str_replace("\n", '', $this->diafan->get_param($_POST, "sales_notes", '', 1))."\n"
			.'manufacturer_warranty='.str_replace("\n", '', $this->diafan->get_param($_POST, "manufacturer_warranty", '', 1))."\n"
			.'country_of_origin='.str_replace("\n", '', $this->diafan->get_param($_POST, "country_of_origin", '', 1));

		return TRUE;
	}

	/**
	 * Сохранение поля "Дополнительные параметры"
	 *
	 * @return boolean true
	 */
	protected function save_param_module()
	{
		if(! $this->site_id)
		{
			$this->get_site_id();
		}
		$price = FALSE;
		$ids = array();

		if($this->diafan->savenew)
		{
			$result = DB::query("SELECT p.id, p.name, p.type FROM {shop_param} as p "
			                    .($this->diafan->configmodules("cat", "shop", $this->site_id)
			                      ? " INNER JOIN {shop_param_category_rel} as cp ON cp.cat_id IN (%d, 0) AND cp.element_id=p.id" : "")
			                    ." WHERE p.trash='0' AND site_id=".$this->site_id
			                    ." GROUP BY p.id", !empty($_POST["cat_id"]) ? $_POST["cat_id"] : ''
			                   );
		}
		else
		{
			DB::query("DELETE FROM {shop_price} WHERE good_id=%d", $this->diafan->save);
			$result = DB::query("SELECT p.id, p.name, p.type FROM {shop_param} as p "
			                    .($this->diafan->configmodules("cat", "shop", $this->site_id)
			                      ? " LEFT JOIN {shop_category_rel} as c ON c.element_id=".$this->diafan->save
			                        ." INNER JOIN {shop_param_category_rel} as cp ON (cp.cat_id=c.cat_id OR cp.cat_id=0) AND cp.element_id=p.id"
			                    : "")
			                    ." WHERE p.trash='0' AND p.site_id=%d GROUP BY p.id"
			                    ." ORDER BY p.sort ASC", $this->site_id
			                   );
		}

		while($row = DB::fetch_array($result))
		{
			if($row["type"] == "multiple")
			{
				DB::query("DELETE FROM {shop_param_element} WHERE param_id=%d AND element_id=%d", $row["id"], $this->diafan->save);
				if(! empty($_POST['check_price_'.$row["id"]]))
				{
					$result_select = DB::query("SELECT id FROM {shop_param_select} WHERE param_id=%d", $row["id"]);
					while ($row_select = DB::fetch_array($result_select))
					{
						$option[] = $row_select["id"];
					}
					foreach($option as $v)
					{
						if(! empty($_POST['param_price_'.$row["id"].'_'.$v]) || ! empty($_POST['param_count_'.$row["id"].'_'.$v]))
						{
							DB::query(
								"INSERT INTO {shop_param_element} (value, param_id, element_id) VALUES ('%s', %d, %d)",
								$v,
								$row["id"],
								$this->diafan->save
							);
							DB::query(
								"INSERT INTO {shop_price} (price, count_goods, param_id, good_id, param_value) VALUES ('%f', %d, %d, %d, '%d')",
								$_POST['param_price_'.$row["id"].'_'.$v],
								$_POST['param_count_'.$row["id"].'_'.$v],
								$row["id"],
								$this->diafan->save,
								$v
							);
						}
					}
					$price = TRUE;
				}
				elseif(! empty($_POST['param'.$row["id"]]) && is_array($_POST['param'.$row["id"]]))
				{
					foreach($_POST['param'.$row["id"]] as $v)
					{
						DB::query(
							"INSERT INTO {shop_param_element} (value, param_id, element_id) VALUES ('%s', %d, %d)",
							$v,
							$row["id"],
							$this->diafan->save
						);
					}
				}

			}
			elseif(! empty($_POST['param'.$row["id"]]))
			{
				$id = 0;
				if(! $this->diafan->savenew)
				{
					$id = DB::query_result("SELECT id FROM {shop_param_element} WHERE param_id=%d AND element_id=%d",
					                       $row["id"], $this->diafan->save);
				}
				if($row["type"] == "date")
				{
					$_POST['param'.$row["id"]] = $this->diafan->unixdate($_POST['param'.$row["id"]]);
				}
				if($id)
				{
					DB::query(
						"UPDATE {shop_param_element} SET ".(in_array($row["type"], array("text", "editor", "textarea")) ?'[value]' : 'value')
						."='%s' WHERE id=%d",
						$_POST['param'.$row["id"]],
						$id
					);
					DB::query("DELETE FROM {shop_param_element} WHERE param_id=%d AND element_id=%d AND id<>%d", $row["id"], $this->diafan->save, $id);
				}
				else
				{
					DB::query(
						"INSERT INTO {shop_param_element} (".(in_array($row["type"], array("text", "editor", "textarea")) ?
										 '[value]' : 'value')
						.", param_id, element_id) VALUES ('%s', %d, %d)",
						$_POST['param'.$row["id"]],
						$row["id"],
						$this->diafan->save
					);
				}
			}
			else
			{
				DB::query("DELETE FROM {shop_param_element} WHERE param_id=%d AND element_id=%d", $row["id"], $this->diafan->save);
			}
			$ids[] = $row["id"];
		}
		DB::query("DELETE FROM {shop_param_element} WHERE param_id NOT IN (".implode(", ", $ids).") AND element_id=%d", $this->diafan->save);
		if(! $price && ! empty($_POST["price"]))
		{
			DB::query(
				"INSERT INTO {shop_price} (price, count_goods, good_id) VALUES ('%f', %d, %d)",
				$_POST['price'],
				$_POST['count'],
				$this->diafan->save
			);
		}
		return TRUE;
	}

	/**
	 * Сопутствующие действия при удалении элемента модуля
	 * @return boolean true
	 */
	protected function delete($del_id, $trash_id)
	{
		$this->del_or_trash_where("shop_price", "good_id=".$del_id, $trash_id);
		$this->del_or_trash_where("shop_rel", "good_id=".$del_id." OR rel_good_id=".$del_id, $trash_id);
		$this->del_or_trash_where("shop_discount_object", "good_id=".$del_id, $trash_id);
		$this->del_or_trash_where("shop_param_element", "element_id=".$del_id, $trash_id);

		return TRUE;
	}
}